/*  Multimètre à cinq fonctions sur ligne série USB.
 A0 est reliée au bouton poussoir de sélection des fonctions.
 A2 est reliée au +5Vcc par 1000 ohms. Calibre 100 ohms à 100 k ohms.
 A3 est reliée au +5Vcc par 100 ohms. Calibre inférieur à 100 ohms.
 A4 est reliée à une résistance de 10 ohms qui va à la masse. (50mA à 500mA)
 A5 est reliée à une résistance de 100 ohms qui va à la masse. (0mA à 50mA) */
 
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define NB_Options 3 // Options indexées entre 1 et 3.

#define N 400 // Nombre de mesures pour faire une moyenne.

#define K10 1.04 // Corrige la valeur du shunt de 1 ohm.
#define K100 1.05 // Corrige la valeur du shunt de 10 ohm.

#define LED_Arduino 13 // LED verte sur le C.I.

// Variables du programme.
boolean Clic_sur_le_BP;
byte Option_du_MENU = 1; // Ordre de l'option en cours.
float Tension_lue, N_Tensions_cumulees, Valeur_calculee, Ancienne_Valeur ;
byte ENTREE;
byte FONCTION = 1; // Fonction en cours.
boolean Entree_A2, Entree_A4;

void setup() {
  Serial.begin(115200);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);
  Affiche_nature_Option_Indexee(); Serial.println();}

void loop() {
  Tester_le_BP_A0(); // Pour détection dans la boucle de base. 
  if (Clic_sur_le_BP) {
    // Prise en compte de l'action sur le B.P.
    Clic_sur_le_BP = false;
    // Traitement de l'option.
    Option_du_MENU++; if (Option_du_MENU > NB_Options) {Option_du_MENU = 1;}
    Affiche_nature_Option_Indexee(); }
  // Traitement de routine de l'option active.
  switch(Option_du_MENU) {
    case 1 : {Mesure_et_affiche_les_TENSIONS(); break;}
    case 2 : {Mesure_et_affiche_les_INTENSITES(); break;}
    case 3 : {Mesure_et_affiche_les_RESISTANCES(); break;} } }
    
void Mesure_et_affiche_les_TENSIONS() {
  N_Tensions_cumulees = 0;
  for (int I = 0; I < (N) ; I++) 
      {Tester_le_BP_A0(); // Pour détection dans la boucle de mesure.
       Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;
  Valeur_calculee = map(Tension_lue, 0, 1023, 0, 5000); // Tension en volts * 1000.
  if (Valeur_calculee != Ancienne_Valeur) {
      Ancienne_Valeur = Valeur_calculee;
      Serial.print("Tension lue sur A1 = ");
      Serial.println(Valeur_calculee/1000); } }

void Mesure_et_affiche_les_INTENSITES() {
  // Mesures sur A4 avec résistance shunt de 10 ohms.
  N_mesures_sur_Ai(4); Entree_A4 = true;
  Valeur_calculee = Tension_lue * K10 / 10;
  if ((Valeur_calculee < 50) && (Valeur_calculee > 0)) {Affiche_Changer_de_calibre();}
  // Si zéro sur A4 mesurer sur A5 avec résistance shunt de 100 ohms.
  if (Valeur_calculee == 0) {
      N_mesures_sur_Ai(5); Entree_A4 = false;
      Valeur_calculee = Tension_lue * K100 / 100;}
  Affiche_Intensite(); }

void Mesure_et_affiche_les_RESISTANCES() {
  // Mesures sur A2 avec résistance shunt de 10 ohms.
  N_mesures_sur_Ai(2);
  Valeur_calculee = Tension_lue * 1000 / (5000 - Tension_lue); 
  if (Valeur_calculee < 100) {Affiche_Changer_de_calibre();}
  if (Tension_lue < 4960)
     {Entree_A2 = true; Affiche_la_resistance();} // N'affiche que si changée.
  // Mesures sur A3 avec résistance de limitation de 100 ohms.
  N_mesures_sur_Ai(3);
  Valeur_calculee = Tension_lue * 100 / (5000 - Tension_lue);
  if (Tension_lue < 4960)
     { Entree_A2 = false; Affiche_la_resistance();} } // N'affiche que si changée.

void Affiche_la_resistance() {  
  if (Valeur_calculee != Ancienne_Valeur) {
  Ancienne_Valeur = Valeur_calculee;
  Serial.print("  Tension lue = ");
  Serial.print(Tension_lue/1000,2);
  Serial.print("   R"); Serial.print(char (233)); Serial.print("sistance sur ");
  if (Entree_A2) {Serial.print("A2");} else {Serial.print("A3");}
  Serial.print(" = ");   Serial.println(Valeur_calculee,1); } }

void N_mesures_sur_Ai(byte ENTREE) { 
  N_Tensions_cumulees = 0;
  for (int I = 0; I < (N) ; I++) 
      {Tester_le_BP_A0(); // Pour détection dans la boucle de mesure.
       Tension_lue = analogRead(ENTREE); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;
  Tension_lue = map(Tension_lue, 0, 1023, 0, 5000); // Tension en volts * 1000.
  if (Tension_lue == 5000) {Tension_lue = 4999.99;} }
  // 4999.99 au lieu de 5000 pour viter une division par zéro dans le calcul de R.
 
 void Affiche_le_mot_Intensite() {Serial.print("Intensit"); Serial.print(char (233));}
 
 void Affiche_Intensite() {  
  if (Valeur_calculee != Ancienne_Valeur) {
  Ancienne_Valeur = Valeur_calculee;
  Serial.print("Tension lue sur ");
  if (Entree_A4) {Serial.print("A4");} else {Serial.print("A5");}
  Serial.print(" = "); Serial.print(Tension_lue/1000);
  Serial.print("  >>>  "); Affiche_le_mot_Intensite(); Serial.print(" = ");
  if (Entree_A4) {Serial.print(Valeur_calculee,0);} else {Serial.print(Valeur_calculee,1);}
  Serial.println(" mA"); } }

void Affiche_Changer_de_calibre() { Serial.println(); Serial.println();
    Serial.println("  Changer de calibre."); Serial.println(); delay(250); }  
  
void Affiche_nature_Option_Indexee() {
    Serial.println();
    switch(Option_du_MENU) {
    case 1 : {Serial.println("Voltmetre 0 a 5Vcc."); break;}
    case 2 : {Serial.println("Amperemetre jusqu'a 500mA."); break;}
    case 3 : {Serial.println("Resistance 0 a 151k Ohms."); break;} }
    Serial.println(); }
  
// >>>>>>>>>>>>> Le B.P. est câblé en logique positive <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 1023.
  byte COMPTEUR;
  if (analogRead(0) > 512){Clic_sur_le_BP = true;
    digitalWrite(LED_Arduino, HIGH); // Allumer le LED verte. 
    // Attendre_la_stabilisation à "1".
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; if (analogRead(0) < 512) {COMPTEUR = 0;} }
    // Attendre_la_stabilisation à "0".
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; if (analogRead(0) > 512) {COMPTEUR = 0;} }
    digitalWrite(LED_Arduino, LOW); } } // Éteindre la LED verte.

  
   
