/* Programme pour mettre au point la fonction COMPTEUR par utilisation
   du TIMER1 en interruptions sur débordement.
   OUI clic long : Sortie de la fonction.
   OUI court : Inverser le sens de transition qui fait compter.
   NON long ou court : Remise à zéro du compteur.   */
   
// Définition des E/S pour ce programme.
#define LED_Arduino 13
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define Entree_Compteur 5 // Entrée D5 pour servira aussi pour mesurer les fréquences.
#define Sortie_PWM 11 // Génération permanente d'un signal carré à 490Hz.

// Variables du programme.
unsigned long Nb_IRQ; // Nombre de débordements du compteur durant le comptage.
unsigned long COMPTEUR, Ancienne_Valeur;
boolean Front_Montant;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;

// Le [Timer 1] de l'ATmega sera notre compteur.
// Ce compteur sur 16 bits déborde à 65536 comptages.
// La variable NB_IRQ comptera le nombre de débordements.
// Elle sera incrémentée par interruptions sur débordement du TIMER1 :
ISR(TIMER1_OVF_vect) {Nb_IRQ++;}

void setup() {
  Serial.begin(115200);
  Serial.println(">>> COMPTEUR <<<"); Serial.println();
  pinMode(Entree_Compteur, INPUT);
  analogWrite(Sortie_PWM, 128);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);}

void loop() {
  Fonction_COMPTEUR();
  Serial.println(); Serial.println("On vient d'en sortir."); Serial.println();
  Serial.println("On y retourne !"); Serial.println();}
  
void Fonction_COMPTEUR() { Serial.println(); Serial.println("FONCTION COMPTEUR en cours.");
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);
  // Autorise les interruptions sur débordement.
  // (Active la procédure ISR(TIMER1_OVF_vect) sur débordement.)
  TIMSK1 = (1<<TOIE1); 
  Clic_sur_le_BP = false; Front_Montant = true;
  RAZ_COMPTEUR(); Augmenter_le_choix = false;
  
  while (!(Augmenter_le_choix && Clic_long)) {
        Tester_entree_A0();
        if (Clic_sur_le_BP) {
           Clic_sur_le_BP = false;
           if (Augmenter_le_choix) 
               {if(!Clic_long) {
                 Front_Montant = !Front_Montant;
                  Ancienne_Valeur++; // Pour forcer l'affichage; 
                  if (Front_Montant) TCCR1B = TCCR1B | (1<<CS10);
                  else  TCCR1B = 6; } }
           else RAZ_COMPTEUR(); }
        //========================= Traiter le COMPTEUR ===============================
        COMPTEUR = TCNT1;
        // Lecture des 16 bits dans la variable COMPTEUR et ajoute le DEBORDEMENT.
        COMPTEUR = TCNT1 + (65536 * Nb_IRQ);
        if (COMPTEUR > 9999999)
           {COMPTEUR = 9999999; TCCR1B = 0;} // Bloquer le compteur.
        // ============================================================================
        if(COMPTEUR != Ancienne_Valeur) {Ancienne_Valeur = COMPTEUR; Afficher_le_compteur();} }
  TIMSK1 = TIMSK1 & 251;
  Clic_long = true; Clic_sur_le_BP = true; }
  
void RAZ_COMPTEUR() {
  COMPTEUR = 0; Ancienne_Valeur = 1; // Pour forcer l'affichage; 
  // Replacer le compteur à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0; 
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);}
  
void Afficher_le_compteur() {
  Serial.print("COMPTEUR de TR");
  if(Front_Montant) Serial.print("+ = "); else Serial.print("- = ");
  Serial.println(COMPTEUR); }
    
 // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_entree_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 1000) Clic_long = true; } }

