/* Programme pour mettre au point le CHRONOMETRE.
   Dans ce PGM la durée "LONGUE" est réduite à 0.5 seconde pour tester.
   ATTENTION : La fonction millis() utilise le TIMER0.
   Donc PWM5 et PWM6 ne sont plus disponibles.
   Ici la fréquence de comptage est calibrée par Tcomptage.
   Elle est très précise car utilise la fonction milli().
   NOTE : La génération de 490Hz sur PWM est active en arrière plan. */
   
// Définition des E/S pour ce programme.
#define LED_Arduino 13
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define Sortie_PWM 11 // Génération permanente d'un signal carré à 490Hz.

// Variables du programme.
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
byte HEURES, MINUTES, SECONDES, DIXIEMES;
int Cadence_de_comptage; // Période du chronomètre exprimée en mS.
unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;
boolean Chrono_ON, Bloquer_a_24H;

void setup() {
  Serial.begin(115200); Clic_sur_le_BP = false; Serial.println();
  Serial.println(">>> CHRONOMETRE <<<"); Serial.println();
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);
  analogWrite(Sortie_PWM, 128);}
  
void loop() {
  Tester_entree_A0();
  if (Clic_sur_le_BP) {
      Clic_sur_le_BP = false; Serial.println();
      Serial.println("On active la fonction chrono."); Serial.println();
      Fonction_CHRONOMETRE(); Serial.println();
      Serial.println("Sortie de la fonction chrono."); Serial.println(); } }

void Fonction_CHRONOMETRE() {
  Clic_long = false; Chrono_ON = false; Bloquer_a_24H = false;
  Cadence_de_comptage = 1000; RAZ_CHRONOMETRE();
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE =========================
  if ((!Bloquer_a_24H) && (Chrono_ON)) Incrementer_le_CHRONOMETRE();
  Tester_entree_A0();
  //-------------- Traiter les deux B.P. --------------
  if (Clic_sur_le_BP) {
     Clic_sur_le_BP = false;
     // B.P. augmenter le choix.
       if (Augmenter_le_choix)
          {Chrono_ON = !Chrono_ON; if (Chrono_ON)
               Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis()+ Cadence_de_comptage; }
     // B.P. diminuer le choix.
       else {if (Clic_long)
                    if (Cadence_de_comptage == 1000) Cadence_de_comptage = 100;
                    else Cadence_de_comptage = 1000;
             else RAZ_CHRONOMETRE(); } } }
  //========================================================================
 }
 
void Traite_Clic_LONG() {
  Serial.print("Clic LONG et cadence = "); Serial.println(Cadence_de_comptage);
  if (Cadence_de_comptage == 1000) Cadence_de_comptage = 10;
  else Cadence_de_comptage = 100;}
 
void Incrementer_le_CHRONOMETRE() {
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = 
      Ancienne_valeur_Temps_Ecoule_depuis_RESET + Cadence_de_comptage;
      if (Cadence_de_comptage == 100) DIXIEMES++; else SECONDES++;
      digitalWrite(LED_Arduino,!digitalRead(LED_Arduino)); // Inverser LED 13.
      if (DIXIEMES == 10) {SECONDES++; DIXIEMES = 0;}
      if (SECONDES == 60) {MINUTES++; SECONDES = 0;}
      if (MINUTES == 60) {HEURES++; MINUTES = 0;}
      Bloquer_a_24H = (HEURES == 24); Afficher_le_CHRONOMETRE(); } }
 
void RAZ_CHRONOMETRE() {
  HEURES = 0; MINUTES = 0; SECONDES = 0; DIXIEMES = 0; 
  Ancienne_valeur_Temps_Ecoule_depuis_RESET = 
    Ancienne_valeur_Temps_Ecoule_depuis_RESET + Cadence_de_comptage;
  Afficher_le_CHRONOMETRE();}
        
void  Afficher_le_CHRONOMETRE() {
  Impose_deux_caracteres(HEURES); Serial.print(HEURES); Serial.print(" H ");
  Impose_deux_caracteres(MINUTES); Serial.print(MINUTES); Serial.print(" Min ");
  Impose_deux_caracteres(SECONDES); Serial.print(SECONDES); Serial.print(" S ");
  if (Cadence_de_comptage == 1000) Serial.println(); else Serial.println(DIXIEMES); }
  
void Impose_deux_caracteres(byte VALEUR) {if (VALEUR < 10) {Serial.print(" ");} }
    
 // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_entree_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 500) Clic_long = true; } }

