/* Ce petit programme permet de développer la fonction fréquencemètre sur D5 avec Arduino.
   La fréquence maximale est celle de l'horloge / 2. (Quartz 16 Mhz.)
   Mais on recommande F/2.5 soit dans notre cas 6,4MHz garantis.
   CHOIX OPERATIONNEL :
   Le B.P. OUI ne fait que permettre de sortir.
   En mesures lentes les B.P. ne sont pris en compte que toutes les secondes.
   Pour permettre d'inverser la rapidité il faut être certain de prendre en
   compte le B.P. NON.
   Vitesse actuelle LENTE : Clic long fait passer en rapide.
   Vitesse actuelle RAPIDE : Clic court fait passer en LENT.
                             Clic long fait passer en HOLD.
   la sortie de HOLD se fait avec un clic sur NON et retour au mode LENT. */

#define LED_Arduino 13 // LED pour montrer la rapidité des mesures.
#define Entree_Compteur 5 // Entrée D5 pour mesurer les fréquences.
#define Sortie_PWM 11 // Génération permanente d'un signal carré à 490Hz.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

// Variables utilisées.
volatile byte Nb_IRQ; // 9Mhz provoquerait 152 Interruptions maximum.
unsigned long VALEUR, Ancienne_valeur; // Fréquence mesurée.
boolean Rapide, HOLD;
int Temps_de_mesure, Affiner_T;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;

// Le [Timer 1] de l'ATmega sera notre compteur.
// Ce compteur sur 16 bits déborde à 65536 comptages.
// Nb_IRQ sera incrémentée par interruptions sur débordement du TIMER1 :
ISR(TIMER1_OVF_vect) {Nb_IRQ++;}

void setup() {
  Serial.begin(115200);
  pinMode(LED_Arduino, OUTPUT);  pinMode(Entree_Compteur,INPUT);
  analogWrite(Sortie_PWM, 128);}

void loop() {
  Serial.println("On active le FREQUENSEMETRE."); Serial.println();
  Fonction_FREQUENCEMETRE(); Serial.println(); delay(1000);
  if (Clic_sur_le_BP) {
      Clic_sur_le_BP = false; Serial.println();
      Serial.println("Sortie du FREQUENSEMETRE."); Serial.println(); } }
 
void Fonction_FREQUENCEMETRE() {
  Clic_long = false; Rapide = false; Clic_sur_le_BP = false;
  Ancienne_valeur = 0;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);	
  // Autorise les interruptions sur débordement.
  // (Active la procédure ISR(TIMER1_OVF_vect) sur débordement définie ci-avant.)
  TIMSK1 = (1<<TOIE1); 
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE ========================= 
  Tester_entree_A0();
  //-------------- Traiter les deux B.P. --------------
  // Le B.P. augmenter le choix n'est utilisé que pour sortir car en
  // mesures à 1Hz il n'est pas pris en compte 9 fois sur 10.
  if ((Clic_sur_le_BP) && (!Augmenter_le_choix))
     {Clic_sur_le_BP = false;
      if ((Rapide) && (Clic_long))
          {Serial.println("Cliquer sur NON pour reprendre."); Rapide = false;
           Serial.print("HOLD >"); 
           Mesurer_la_frequence(); Afficher_la_frequence();
           while (!Clic_sur_le_BP) Tester_entree_A0(); Clic_sur_le_BP = false;
           Ancienne_valeur = 0;}
      else Rapide = !Rapide;}
  //---------------------------------------------------    
  Mesurer_la_frequence();
  if (VALEUR != Ancienne_valeur) { Ancienne_valeur = VALEUR; Afficher_la_frequence();} }
  //========================================================================
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir.

void  Mesurer_la_frequence() { 
  digitalWrite(LED_Arduino, !digitalRead(LED_Arduino)); // Inverser la LED 13.
  if (Rapide) {Temps_de_mesure = 99;  Affiner_T = 986;}
     else {Temps_de_mesure = 999; Affiner_T = 978;}
  // Replacer le compteur à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0;
  // Par définition, la fréquence correspond aux nombres d'impulsions par seconde.
  // Pendant ce délai il y a comptage sur Timer 1 et éventuellement plusieurs débordements.
  //============= Calibrer la durée de mesures ===============
  delay(Temps_de_mesure); 
  delayMicroseconds(Affiner_T);
  // =========================================================  
  // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
  VALEUR = TCNT1;
  // Lecture des 16 bits dans la variable COMPTEUR et ajoute le DEBORDEMENT.
  VALEUR = TCNT1 + (65536 * Nb_IRQ);
  // =========================================================
  if(Rapide) VALEUR = VALEUR * 10;}
  
void  Afficher_la_frequence() {  
  Serial.print(" Fr"); Serial.print(char (233)); Serial.print("quence = ");
  Serial.print(VALEUR); Serial.print("Hz soit une periode de : ");
  if (VALEUR > 0) VALEUR = 1000000 / VALEUR; else VALEUR = 0; // Attention à la division par zéro !
  Serial.print(VALEUR); Serial.print(char (181)); Serial.println("S."); }
  
          // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_entree_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 1000) Clic_long = true; } }


