/* Ce petit programme permet de développer la fonction Périodemètre sur D5 avec Arduino.
   CHOIX OPERATIONNEL :
   Le B.P. OUI ne fait que permettre de sortir.
   Le B.P. NON court ou long fait passer en PAUSE.  */

#define LED_Arduino 13 // LED pour montrer la rapidité des mesures.
#define Entree_SIGNAL 5 // Entrée D5 pour mesurer les fréquences.
#define Sortie_PWM 11 // Génération permanente d'un signal carré à 490Hz.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

// Variables utilisées.
volatile byte Nb_IRQ;
unsigned long VALEUR, Ancienne_valeur; // Fréquence mesurée.
float Valeur_cumulee;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
byte Option_du_MENU;
boolean ESPACE; // Signale qu'il faut compléter avec des espaces et non des "0".
unsigned int PAQUET; // Valeur numérique comprise entre 0 et 999.

// Le [Timer 1] de l'ATmega sera notre chronomètre rapide.
// Ce compteur sur 16 bits déborde à 65536 comptages.
// Nb_IRQ sera incrémentée par interruptions sur débordement du TIMER1 :
ISR(TIMER1_OVF_vect) {Nb_IRQ++;}

void setup() {
  Serial.begin(115200);
  pinMode(LED_Arduino, OUTPUT);  pinMode(Entree_SIGNAL,INPUT);
  analogWrite(Sortie_PWM, 128);}

void loop() {
  Serial.println("On active le PERIODEMETRE."); Serial.println();
  Fonction_PERIODEMETRE(); Serial.println(); delay(1000);
  if (Clic_sur_le_BP) {
      Clic_sur_le_BP = false; Serial.println();
      Serial.println("Sortie du PERIODEMETRE."); delay(1000); } }
 
void Fonction_PERIODEMETRE() {
 Serial.println(); Serial.println("Fonction PERIODEMETRE."); Serial.println();
  analogWrite(Sortie_PWM, 128); Clic_sur_le_BP = false;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  Valeur_cumulee = 0;
  TCCR1B = 0; // Bloquer VALEUR.
  //==================== MESURER ======================
  for (byte I=0; I<10; I++) { // Dix mesures pour lisser par calcul de moyenne.
    VALEUR = 0; Nb_IRQ=0; TCNT1 = 0; // Forcer le compteur rapide à zéro.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 0; // Bloquer VALEUR.
    // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
    VALEUR = TCNT1;
    // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
    VALEUR = TCNT1 + (65536 * Nb_IRQ);
    Valeur_cumulee = Valeur_cumulee + (VALEUR / 16);} 
    VALEUR = Valeur_cumulee / 10; // Calculer la moyenne des dix mesures. 
  //==================== AFFICHER =====================
  if (VALEUR != Ancienne_valeur) {Ancienne_valeur = VALEUR; Afficher_la_Periode();}    
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {if (!Augmenter_le_choix) PAUSE(); } }
  //=============================================================================
  pinMode(Sortie_PWM, INPUT); Option_du_MENU = 1;
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir. 
  
void  Afficher_la_Periode() {  Serial.print(" P"); Serial.print(char (233));
  Serial.print("riode = "); Affiche_VALEUR_avec_formatage(); Serial.print(" ");
  Serial.print(char(181)); Serial.print("S soit une Fr"); Serial.print(char (233));
  Serial.print("quence = ");
//  if (VALEUR < 10000) Serial.print((float)(1000000 / VALEUR));
if (VALEUR <= 2000) Serial.print(1000000 / VALEUR); // QRG >= à 500Hz.
  else {VALEUR = 1000000000 / VALEUR; Affiche_VALEUR_avec_formatage();}
  Serial.println(" Hz.");
  if (VALEUR < 1000) {Serial.print("Pr"); Serial.print(char(233));
      Serial.print("f"); Serial.print(char(233));
      Serial.println("rer le FREQUENCEMETTRE.");} }
      
void PAUSE() {Clic_sur_le_BP = false; Serial.println();
  Serial.println("PAUSE en cours.");
  Serial.print("B.P. [Fonction moins] pour reprendre : ");
  while ((!Clic_sur_le_BP) || ( Augmenter_le_choix)) Tester_le_BP_A0();
  Clic_sur_le_BP = false; Serial.println("Sortie de la PAUSE.");}
  
void COMPLETER() {if (ESPACE) {Serial.print(" ");} else {Serial.print("0");} } 
  
void Afficher_un_PAQUET() {// Valeur comprise entre 0 et 999.
  if (PAQUET < 100) {COMPLETER();} // Afficher un espace ou un zéro.
  if (PAQUET < 10) {COMPLETER();}  // Afficher un autre espace ou un autre zéro.
  if (PAQUET == 0) {COMPLETER();} else {Serial.print(PAQUET);} }

void Affiche_VALEUR_avec_formatage() {
  // Formate l'affichage pour placer les points et les espaces en tête pour
  // 9 caractères. Ne modifie pas la variable VALEUR.
  unsigned long SAV_VALEUR = VALEUR; boolean Mega_non_nul;
  ESPACE = true; // Commencer par des espaces si valeur insuffisante.
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les MEGA <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000000; // Isoler les Méga dans PAQUET et VALEUR inchangée.
  // Si > 0 afficher les Méga suivi d'un point, si non afficher deux espaces.
  if (PAQUET == 0) {Serial.print("  "); Mega_non_nul = false;}
      else {Serial.print(PAQUET); Serial.print("."); ESPACE = false; Mega_non_nul = true;
            // Enlever les MEGA déjà affichés dans VALEUR :
            VALEUR = VALEUR - (PAQUET * 1000000); }
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les KILO <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000; // Isoler les KILO dans PAQUET et VALEUR inchangée.
  Afficher_un_PAQUET();
  if ((PAQUET > 0) | (Mega_non_nul))
     {Serial.print("."); ESPACE = false;} else {Serial.print(" "); } 
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les UNITÉS <<<<<<<<<<<<<<<<<<<<<<<<<<*/
  // Enlever les KILO déjà affichés à VALEUR.
  if (PAQUET > 0) {PAQUET = VALEUR - (PAQUET * 1000);} else {PAQUET = VALEUR;}
  if ((PAQUET == 0) && (ESPACE)) {Serial.print("  0");} else {Afficher_un_PAQUET();}
  VALEUR = SAV_VALEUR; }
  
          // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 1000) Clic_long = true; } }


