/* Programme d'expérimentation de mesure d'impulsions.
   Les fonctions pulseIn() de l'IDE d'Arduino n'ont pas donné de bons
   résultats en termes de précision, raison pour laquelle les mesures
   sont réalisées par comptage avec TIMER1 de l'horloge XTAL avec IRQ.
   L'entrée sera mesurée sur D5 car BINAIRE et entrée générale sur
   le petit laboratoire polyvalent.
   NON fait passer en PAUSE ou reprise des mesures.
   OUI "long" fait sortir.
   La plage de mesure s'étend d'environ 1Hz à 1000Hz à 0.5% de précision.
   On peut mesurer jusqu'à 100kHz mais avec des fluctuations. Dans ce cas
      il faut passer en PAUSE et analyser les affichages.
   Attension à 1Hz le temps de mesure est de 10 secondes. */

#define Entree_SIGNAL 5 // Entrée D5 pour mesurer les impulsions.
#define Sortie_PWM 11 // Génération "permanente" d'un signal carré à 490Hz.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define LED_Arduino 13 // LED verte sur le C.I.

// Variables utilisées.
float Valeur_cumulee;
unsigned long VALEUR, Ancienne_valeur; // Impulsion mesurée.
unsigned long Pulse_plus; // Impulsion positive mémorisée.
unsigned long Pulse_moins; // Impulsion négative mémorisée.
float Rapport_cyclique;
byte Option_du_MENU;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
volatile byte Nb_IRQ; // Le VALEUR sera bloqué à 152 Interruptions.
boolean ESPACE; // Signale qu'il faut compléter avec des espaces et non des "0".
unsigned int PAQUET; // Valeur numérique comprise entre 0 et 999.

ISR(TIMER1_OVF_vect) {Nb_IRQ++;} //Routine d'interruption sur débordement du TIMER1.

void setup() {
  Serial.begin(115200); pinMode(Entree_SIGNAL, INPUT);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);}

void loop() {
  Fonction_IMPULSIOMETRE();
  Serial.println(); Serial.println("On vient d'en sortir."); Serial.println();
  delay(1000);
  Serial.println("On y retourne !"); Serial.println(); }
  
void Fonction_IMPULSIOMETRE() {
  Serial.println(); Serial.println("Fonction IMPULSIOMETRE."); Serial.println();
  analogWrite(Sortie_PWM, 128); Clic_sur_le_BP = false;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple VALEUR.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  Valeur_cumulee = 0;
  //==================== MESURER ======================
  for (byte I=0; I<10; I++) { // Dix mesures pour lisser par calcul de moyenne.
    TCCR1B = 0; // Bloquer VALEUR.
    RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
    TCCR1B = 0; // Bloquer VALEUR.
    VALEUR = TCNT1;
    // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
    VALEUR = TCNT1 + (65536 * Nb_IRQ);
    Valeur_cumulee = Valeur_cumulee + VALEUR / 16; }
  Pulse_plus = VALEUR / 16;
  Valeur_cumulee = 0;
  for (byte I=0; I<10; I++) {
    TCCR1B = 0; // Bloquer VALEUR.
    RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant. 
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
    TCCR1B = 0; // Bloquer VALEUR.
    VALEUR = TCNT1;
    // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
    VALEUR = TCNT1 + (65536 * Nb_IRQ);
    Valeur_cumulee = Valeur_cumulee + VALEUR / 16; }
  Pulse_moins = VALEUR / 16;
  //==================== AFFICHER =====================
  if ((Pulse_moins + Pulse_plus) != Ancienne_valeur)
     {Ancienne_valeur = Pulse_moins + Pulse_plus; Afficher_Impulsion();}
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
      if (!Augmenter_le_choix) PAUSE(); } }
  //=============================================================================
  pinMode(Sortie_PWM, INPUT); Option_du_MENU = 1;
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir. 
  
void RAZ_VALEUR() {
  VALEUR = 0; Ancienne_valeur = 1; // Pour forcer l'affichage; 
  // Replacer le VALEUR à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0; 
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);}
  
  void Afficher_Impulsion() {
  Serial.print(" PL+ = "); VALEUR = Pulse_plus;
  Affiche_VALEUR_avec_formatage(); Serial.print(char(181));
  Serial.print("S PL- = "); VALEUR = Pulse_moins;
  Affiche_VALEUR_avec_formatage(); Serial.print(char(181));
  Serial.print("S T = "); VALEUR = Pulse_moins + Pulse_plus;
  Affiche_VALEUR_avec_formatage(); Serial.print(char(181));
  Serial.print("S QRG = "); Serial.print(1000000 / (Pulse_moins + Pulse_plus));
  Rapport_cyclique = ((float)Pulse_plus /(Pulse_moins + Pulse_plus));
  Serial.print("Hz PL+/T = "); Serial.println(Rapport_cyclique,2); }
  
void PAUSE() {Clic_sur_le_BP = false; Serial.println();
  Serial.println("PAUSE en cours.");
  Serial.print("B.P. [Fonction moins] pour reprendre : ");
  while ((!Clic_sur_le_BP) || ( Augmenter_le_choix)) Tester_le_BP_A0();
  Clic_sur_le_BP = false; Serial.println("Sortie de la PAUSE.");}
  
void COMPLETER() {if (ESPACE) {Serial.print(" ");} else {Serial.print("0");} } 
  
void Afficher_un_PAQUET() {// Valeur comprise entre 0 et 999.
  if (PAQUET < 100) {COMPLETER();} // Afficher un espace ou un zéro.
  if (PAQUET < 10) {COMPLETER();}  // Afficher un autre espace ou un autre zéro.
  if (PAQUET == 0) {COMPLETER();} else {Serial.print(PAQUET);} }

void Affiche_VALEUR_avec_formatage() {
  // Formate l'affichage pour placer les points et les espaces en tête pour
  // 9 caractères. Ne modifie pas la variable VALEUR.
  unsigned long SAV_VALEUR = VALEUR; boolean Mega_non_nul;
  ESPACE = true; // Commencer par des espaces si valeur insuffisante.
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les MEGA <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000000; // Isoler les Méga dans PAQUET et VALEUR inchangée.
  // Si > 0 afficher les Méga suivi d'un point, si non afficher deux espaces.
  if (PAQUET == 0) {Serial.print("  "); Mega_non_nul = false;}
      else {Serial.print(PAQUET); Serial.print("."); ESPACE = false; Mega_non_nul = true;
            // Enlever les MEGA déjà affichés dans VALEUR :
            VALEUR = VALEUR - (PAQUET * 1000000); }
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les KILO <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000; // Isoler les KILO dans PAQUET et VALEUR inchangée.
  Afficher_un_PAQUET();
  if ((PAQUET > 0) | (Mega_non_nul))
     {Serial.print("."); ESPACE = false;} else {Serial.print(" "); } 
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les UNITÉS <<<<<<<<<<<<<<<<<<<<<<<<<<*/
  // Enlever les KILO déjà affichés à VALEUR.
  if (PAQUET > 0) {PAQUET = VALEUR - (PAQUET * 1000);} else {PAQUET = VALEUR;}
  if ((PAQUET == 0) && (ESPACE)) {Serial.print("  0");} else {Afficher_un_PAQUET();}
  VALEUR = SAV_VALEUR; }
  
// >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 500) Clic_long = true; } }
