/* Programme d'expérimentation générateur PWM.
   OUI long : Sortie de la fonction bouclée.
   OUI court : Permuter au mode suivant. ESCALIER > RAMPE > MANUEL ANALOGIQUE.
   
   NON Court ou Long :
       Mode MANUEL >   Variation Prograde ou Rétrograde,
       Mode ESCALIER > Incrémenter les "marches".
                      (11 échelons compris entre 3 et 253)
       Mode RAMPE >   Rampe Rapide ou Lente. 
       
   NOTE : PWM restrainte à [1 - 254] pour générer en permanence
          du signal et exclure "0" et "1" permanents.   */
   
#define Sortie_PWM 11 // Génération "permanente" d'un signal PWM à 490Hz.
#define MESURE 1 // A1 utilisée car A0 réservée aux deux B.P.
#define LED_Arduino 13 // LED verte sur le C.I.   
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

// Variables utilisées.
int N = 400; // Nombre de mesures pour faire une moyenne.
byte Option_du_MENU = 1; // Ordre de l'option en cours.
boolean Rapide;
float Valeur_calculee, Ancienne_Valeur;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
float Tension_lue,N_Tensions_cumulees ;
unsigned int PAQUET; // Contiendra la valeur numérisée.
boolean Front_Montant; // Ici indique une variation PROGRADE en mode manuel.

void setup() {
  Serial.begin(115200); pinMode(MESURE, INPUT);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);}

void loop() {
  Fonction_GENERATEUR_PWM();
  Serial.println(); Serial.println("On vient d'en sortir."); Serial.println();
  delay(1000);
  Serial.println("On y retourne !"); Serial.println(); }
  
void Fonction_GENERATEUR_PWM() {
  Front_Montant = true; // variation PROGRADE en mode manuel.
  Option_du_MENU = 1; // Mode de génération en cours.
  Serial.println(); Serial.println("Fonction GENERATEUR PWM."); Serial.println();
  Clic_sur_le_BP = false; Clic_long = false; Augmenter_le_choix = false;
  Ancienne_Valeur = 2000; // Forcer le premier affichage. 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  N_Tensions_cumulees = 0;
  for (int I = 0; I < (N) ; I++) 
      {Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;
  //=========== Traiter L'OPTION manuelles =============
  if (Option_du_MENU == 1) { 
      Valeur_calculee = map(Tension_lue, 0, 1023, 1, 254);
      if(!Front_Montant) Valeur_calculee = 254 - Valeur_calculee;
      if(Valeur_calculee != Ancienne_Valeur) {Ancienne_Valeur = Valeur_calculee;
      analogWrite(Sortie_PWM,Valeur_calculee);  // Modification du rapport cyclique.
      //================= AFFICHER ==================
      PAQUET = (int)Tension_lue; 
      if (Front_Montant) Serial.print("(Pro");
      else {Serial.print("(R"); Serial.print(char(233)); Serial.print("tro");}   
      Serial.print("rade ) Commande = ");
      if (PAQUET < 10)  Serial.print(" ");
      if (PAQUET < 100)  Serial.print(" ");
      if (PAQUET < 1000)  Serial.print(" ");
      Serial.print(PAQUET);
      Serial.print("  Valeur PWM = ");
      if (Valeur_calculee < 10)  Serial.print(" ");
      if (Valeur_calculee < 100)  Serial.print(" ");  
      Serial.print(Valeur_calculee,0);
      Serial.print("  Rapport cyclique = ");
      Serial.println(Valeur_calculee / 254,2); } }
      
  //=============== Traiter l'ESCALIER =================
  if (Option_du_MENU == 2) {Valeur_calculee = 128;
     analogWrite(Sortie_PWM,Valeur_calculee); // Impulsions PL+ moyenne.
     Serial.println(); Serial.println(" Mode ESCALIER en cours.");
     Afficher_PWM();
     Clic_sur_le_BP = false; Augmenter_le_choix = false; Clic_long = false;
     //--------- Tant que NON passage à RAMPE ----------
     while (!(Augmenter_le_choix)) {
       Tester_le_BP_A0();
       if ((Clic_sur_le_BP) && (!Augmenter_le_choix))  
         // Monter d'une marche. (11 marches comprises entre 3 et 253)
         {if (Valeur_calculee == 253) Valeur_calculee = 3; else Valeur_calculee = Valeur_calculee + 25; 
          analogWrite(Sortie_PWM,Valeur_calculee); Clic_sur_le_BP = false;
          Afficher_PWM(); }
     //------------- Passage au mode RAMPE -------------
       if (Augmenter_le_choix)
          {Clic_sur_le_BP = false; Option_du_MENU = 3;} } }
        
  //================ Traiter la RAMPE ================== 
  if ((Option_du_MENU == 3) && (!Clic_long)) {
     Afficher_mode_RAMPE();
     Clic_sur_le_BP = false; Augmenter_le_choix = false; Clic_long = false;
     //-------- Tant que NON retour à MANUEL ----------
     while (!(Augmenter_le_choix)) {
       //-------------- générer la RAMPE --------------
       if (Rapide) delay(10); else delay(30);
       Valeur_calculee++; if (Valeur_calculee == 255) Valeur_calculee = 1;
       analogWrite(Sortie_PWM,Valeur_calculee);
       //-------------- Traiter les B.P. --------------
       Tester_le_BP_A0();
       if ((Clic_sur_le_BP) && (!Augmenter_le_choix))  
         // Inverser le boolée Rapide.)
         {Rapide = !Rapide; Clic_sur_le_BP = false; Afficher_mode_RAMPE(); } 
     //------------- Retour au mode MANUEL -------------
     if (Augmenter_le_choix)
        {Clic_sur_le_BP = false; Option_du_MENU = 1;
        Serial.println(" Retour au mode manuel."); Serial.println();
     Ancienne_Valeur = 2000;} } } // Forcer le premier affichage. 
  //====================================================    
  // traiter les deux B.P. dans la Fonction_GENERATEUR_PWM :
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     // B.P. augmenter le choix.
       if ((Augmenter_le_choix) && (!Clic_long))
          {Clic_sur_le_BP = false; Option_du_MENU = 2;}
     // B.P. diminuer le choix.
     if (!Augmenter_le_choix)
        {Front_Montant = !Front_Montant; Clic_sur_le_BP = false; } } }
  //=============================================================================
  pinMode(Sortie_PWM, INPUT); Option_du_MENU = 1; }
  
void Afficher_PWM() {Serial.print(" Echelon PWM = "); Serial.println(Valeur_calculee);}

void Afficher_mode_RAMPE() {
  Serial.println(); Serial.print(" Mode RAMPE ");
  if (Rapide) Serial.print("RAPIDE"); else Serial.print("LENTE");
  Serial.println(" en cours."); }

// >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 500) Clic_long = true; } }
