/* Programme d'expérimentation de mesure de condensateurs.
   Les mesures de la constante de temps sont réalisées par comptage
   avec TIMER1 de l'horloge XTAL avec utilisation des interruptions
   sur débordement. Pour les mesures il faut ponter A1 à D12.
   Le condensateur est placé entre l'entrée (Coté +) et la masse. (Coté -)
   OUI "long" fait sortir.
   OUI "court" est ignoré.
   NON fait passer en PAUSE. (Long ou court.)
   La plage de mesure rapide va de 10µF à 10000µF. (Voir plus de 10000µF.)
   La plage de mesure lente va de 100µF à 10000µF.
   La décharge du condensateur est plus ou moins longue. La fréquence des mesures
   diminue quand la capacité mesurée augmente. */

#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define LED_Arduino 13 // LED verte sur le C.I.
#define Sortie_SIGNAL 12 // Generation du signal étalon sur D12.

// Variables utilisées.
byte Option_du_MENU;
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
unsigned long VALEUR, Ancienne_valeur; // Fréquence mesurée.
volatile byte Nb_IRQ; // VALEUR sera bloqué à 152 Interruptions.
unsigned long Pulse_plus; // Utilisé ici pour mesurer A1.
float Tension_lue, N_Tensions_cumulees, Valeur_calculee, Ancienne_Valeur ;

ISR(TIMER1_OVF_vect) {Nb_IRQ++;} // Routine d'interruption sur débordement du TIMER1.

void setup() {
  Serial.begin(115200); pinMode(Sortie_SIGNAL, OUTPUT);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);}

void loop() {
  Fonction_CAPACIMETRE();
  Serial.println(); Serial.println("On vient d'en sortir."); Serial.println();
  delay(1000);
  Serial.println("On y retourne !"); Serial.println(); }
  
void Fonction_CAPACIMETRE() {
  Serial.println(); Serial.println("Fonction CAPACIMETRE."); Serial.println();
  Clic_sur_le_BP = false; Ancienne_Valeur = 12345; // Forcer l'affichage.
  TCCR1A = 0;   // Le Timer 1 est configuré en simple VALEUR.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  //==================== MESURER ======================
  TCCR1B = 0; // Bloquer VALEUR.
  RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant.
  Valeur_calculee = 0; Pulse_plus = 0; // Pulse_plus mesure A1.
  digitalWrite(Sortie_SIGNAL,HIGH); // D12 passe à +5vcc.
  TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
  while (Pulse_plus < 644) {Pulse_plus = analogRead(1);}
  TCCR1B = 0; // Bloquer VALEUR.
  digitalWrite(Sortie_SIGNAL,LOW); // D12 repasse à zéro pour décharger le condensateur.
  VALEUR = TCNT1;
  // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
  VALEUR = TCNT1 + (65536 * Nb_IRQ);
  Valeur_calculee = VALEUR / 4450;
  //==================== AFFICHER =====================
  if (Pulse_plus > 1000) Valeur_calculee = 0; // Afficher zéro si pas de condensateur.
  if ((Valeur_calculee) != Ancienne_Valeur)
     {Ancienne_Valeur = Valeur_calculee; Afficher_condensateur();}
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     if (!Augmenter_le_choix) {Clic_sur_le_BP = false; PAUSE();
         Ancienne_Valeur = 12345; } } // Forcer l'affichage en sortie de PAUSE.
  // Ci-dessous assurer la décharge complète du condensateur.
  while (Pulse_plus > 10) {Pulse_plus = analogRead(1);} }
  //=============================================================================
  Option_du_MENU = 1;
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir.
  
void PAUSE() {
  Clic_sur_le_BP = false; Serial.println();
  Serial.println("PAUSE : NON pour reprendre >");
  while (!Clic_sur_le_BP) Tester_le_BP_A0(); Clic_sur_le_BP = false;}
  
void Afficher_condensateur() {
  Serial.print("Capacite = "); Serial.print(Valeur_calculee,0);
  Afficher_micro(); Serial.println("F."); }
  
void  Afficher_micro() {Serial.print(" "); Serial.print(char(181));}
  
void RAZ_VALEUR() {
  VALEUR = 0; Ancienne_valeur = 1; // Pour forcer l'affichage; 
  // Replacer le VALEUR à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0; 
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);}
  
// >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 500) Clic_long = true; } }
