 /*  MINI LABORATOIRE avec affichage sur la ligne série USB.
 Ce programme constitue une synthèse des modules d'expérimentation.
 Mis à part le voltmètre à rampe analogique, il préfigure le multimètre
     polyvalent à affichage sur écran LCD.
 A0 est reliée au bouton poussoir de sélection des fonctions.
 A1 est l'entrée "générale" pour la mesure des tensions.
 A2 est reliée au +5Vcc par 1000 ohms. Calibre 100 ohms à 100 k ohms.
 A3 est reliée au +5Vcc par 100 ohms. Calibre inférieur à 100 ohms.
 A4 est reliée à une résistance de 10 ohms qui va à la masse. (50mA à 500mA)
 A5 est reliée à une résistance de 100 ohms qui va à la masse. (0mA à 50mA)
 D5 est réservée à la mesure des fréquences et des périodes.
 D11 est en sortie d'un signal PWM fournissant un signal carré de 490 Hz.
 D12 sert de sortie pour les signaux BF / ETALON et continuité.
 Dans ce PGM la durée "LONGUE" est réduite à 0.5 seconde.                    */
 
 /* ================ Fonctions actuellement intégrées ===================
  Voltmètre pour tensions continues. (Deux calibres dont 0 à 50Vcc.)
  Voltmètre pour tensions alternatives. (Deux calibres dont 30 à 60Vcc.)
  AMPEREMETRE
  OHMMETRE
  COMPTEUR
  CHRONOMETRE
  FREQUENCEMETRE
  PERIODEMETRE
  IMPULSIOMETRE
  GENERATEUR BF
  GENERATEUR ETALON
  GENERATEUR PWM
  Pilotage de SERVOMOTEURS
  TESTEUR DE CONTINUITE
  CAPACIMETRE 
  */

#define Version_PGM "Logiciel FE6AAE - Version 2 du 29 avril 2015."  
// Le croquis utilise 17 590 octets soit 54% de l'espace de stockage de programmes.

#define Entree_Frequencemetre 5 // Entrée D5 pour la mesure des fréquences.
#define Entree_Compteur 5 // Entrée D5 pour servira aussi pour mesurer les fréquences.
#define Entree_SIGNAL 5 // Entrée D5 pour mesurer les impulsions.
#define Sortie_PWM 11 // Génération "permanente" d'un signal carré à 490Hz.
#define Sortie_SIGNAL 12 // Generation du signal étalon sur D12.
#define LED_Arduino 13 // LED verte sur le C.I.

#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define NB_Options 17 // Options indexées entre 1 et 17.
#define MAX_QRG_ETALON 18 // Nombre de fréquences possibles situées entre 1 et 18.

#define K10 1.04 // Corrige la valeur du shunt de 1 ohm.
#define K100 1.05 // Corrige la valeur du shunt de 10 ohm.

// Variables du programme.
int N = 400; // Nombre de mesures pour faire une moyenne.
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
boolean Confirmation; // Un choix est confirmé si égal à true.
byte Option_du_MENU = 1; // Ordre de l'option en cours.
float Tension_lue, N_Tensions_cumulees, Valeur_calculee, Ancienne_Valeur;
float Valeur_cumulee;
byte ENTREE; byte Valeur_PWM, Ancienne_PWM;
byte QRG_ETALON; // Numéro de la fréquence à générer.
byte FONCTION = 1; // Fonction en cours.
boolean Entree_A2, Entree_A4;
byte HEURES, MINUTES, SECONDES, DIXIEMES;
int Cadence_de_comptage; // Période du chronomètre exprimée en mS.
unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;
unsigned long CNA; // Mesure analogique retournée par le CNA.
unsigned long Pulse_plus; // Impulsion positive mémorisée.
unsigned long Pulse_moins; // Impulsion négative mémorisée.
float Rapport_cyclique;
boolean Chrono_ON, Bloquer_a_24H;
volatile byte Nb_IRQ; // Le COMPTEUR sera bloqué à 152 Interruptions.
//unsigned long COMPTEUR, Ancien_compteur;
unsigned long VALEUR, Ancienne_valeur; // Fréquence mesurée.
boolean Rapide;
boolean Front_Montant;
int Temps_de_mesure, Affiner_T;
boolean ESPACE; // Signale qu'il faut compléter avec des espaces et non des "0".
unsigned int PAQUET; // Valeur numérique comprise entre 0 et 999.

ISR(TIMER1_OVF_vect) {Nb_IRQ++;} //Routine d'interruption sur débordement du TIMER1.

void setup() {
  Serial.begin(115200);
  pinMode(LED_Arduino, OUTPUT); digitalWrite(LED_Arduino, LOW);
  pinMode(Entree_Compteur, INPUT); // Également valable pour Entree_Frequencemetre.
  pinMode(Sortie_SIGNAL, OUTPUT); Serial.println();
  Serial.println(Version_PGM);
  Serial.println(); Affiche_nature_Option_Indexee();
  analogWrite(Sortie_PWM, 128);} // génération PWM en option par défaut.

void loop() {
  Tester_le_BP_A0(); // Pour détection dans la boucle de base. 
  if (Clic_sur_le_BP) {
    // ===== Traiter la requête de l'un des deux B.P. =====
    Clic_sur_le_BP = false; // Action sur le B.P. prise en compte.
    // Traitement de l'option.
    if (Augmenter_le_choix)
         {Option_du_MENU++; if (Option_du_MENU > NB_Options) {Option_du_MENU = 1; } }
    else {Option_du_MENU--; if (Option_du_MENU == 0) {Option_du_MENU = NB_Options; } }
    if (Clic_long) { Option_du_MENU = 1; }
    Affiche_nature_Option_Indexee(); }
  // ===== Traitement de routine de l'option active ici. =====
  switch(Option_du_MENU) {
    case 1 : {Mesure_les_TENSIONS(); 
              Valeur_calculee = map(Tension_lue, 0, 1021, 0, 5000);
              // Tension en volts * 1000. Valeur 1021 au lieu de 1023
              // pour compenser la 330kOhm du calibre 50Vcc.
              Affiche_les_TENSIONS(); break;}
    case 2 : {Mesure_les_TENSIONS();
              Valeur_calculee = map(Tension_lue, 0, 1021, 0, 5000);
              Affiche_les_TENSIONS(); break;}
    case 3 : {Mesure_les_TENSIONS();
              Valeur_calculee = map(Tension_lue, 0, 1021, 0, 3280); // 3280 pour 30v.
              if(Valeur_calculee < 3100) Affiche_les_TENSIONS(); // U < 31V.
              else Serial.println(" U > 30V changer de calibre."); break;}
    case 4 : {Mesure_les_TENSIONS();
              Valeur_calculee = map(Tension_lue, 0, 1021, 0, 5842); // 5842 pour 60v.
              if(Valeur_calculee > 2500) Affiche_les_TENSIONS(); // U < 25V.
              else Serial.println(" U < 30V changer de calibre."); break;}              
    case 5 : {Mesure_et_affiche_les_INTENSITES(); break;}
    case 6 : {Mesure_et_affiche_les_RESISTANCES(); break;}
    case 7 : {CONFIRMER(); if (Confirmation) Fonction_COMPTEUR(); break;}
    case 8 : {CONFIRMER(); if (Confirmation) Fonction_CHRONOMETRE(); break;}
    case 9 : {CONFIRMER(); if (Confirmation) Fonction_FREQUENCEMETRE(); break;}
    case 10 : {CONFIRMER(); if (Confirmation) Fonction_PERIODEMETRE(); break;}
    case 11 : {CONFIRMER(); if (Confirmation) Fonction_IMPULSIOMETRE(); break;}
    case 12 : {CONFIRMER(); if (Confirmation) Fonction_GENERATEUR_BF(); break;}
    case 13 : {CONFIRMER(); if (Confirmation) Fonction_ETALON_de_QRG(); break;}
    case 14 : {CONFIRMER(); if (Confirmation) Fonction_GENERATEUR_PWM(); break;}
    case 15 : {CONFIRMER(); if (Confirmation) Fonction_pilotage_des_SERVOMOTEURS(); break;}
    case 16 : {CONFIRMER(); if (Confirmation) Fonction_TESTEUR_de_CONTINUITE(); break;}    
    case 17 : {CONFIRMER(); if (Confirmation) Fonction_CAPACIMETRE();} } }

void Affiche_nature_Option_Indexee() {
    Serial.println();
    switch(Option_du_MENU) {
      case 1 : {Affiche_le_mot_VOLTMETRE(); Serial.println(" 0 a 5Vcc."); break;}
      case 2 : {Affiche_le_mot_VOLTMETRE(); Serial.println(" 0 a 50Vcc."); break;}
      case 3 : {Affiche_le_mot_VOLTMETRE(); Serial.println(" 3 a 30V alt."); break;}
      case 4 : {Affiche_le_mot_VOLTMETRE(); Serial.println(" 30 a 60V alt."); break;}
      case 5 : {Serial.print("Amp"); Serial.print(char(232)); Serial.print("rem");
                Serial.print(char(232)); Serial.println("tre jusqu'a 500mA."); break;}
      case 6 : {Serial.print("R"); Serial.print(char(233));
                Serial.println("sistance 0 a 151k Ohms."); break;}
      case 7 : {Serial.print("COMPTEUR"); break;}
      case 8 : {Serial.print("CHRONOMETRE"); break;}
      case 9 : {Serial.print("FREQUENCEMETRE"); break;}
      case 10 : {Serial.print("PERIODEMETRE"); break;}
      case 11 : {Serial.print("IMPULSIOMETRE"); break;}
      case 12 : {Serial.print("GENERATEUR BF"); break;}
      case 13 : {Serial.print("GENERATEUR ETALON"); break;}
      case 14 : {Serial.print("GENERATEUR_PWM"); break;}
      case 15 : {Serial.print("Pilotage de SERVOMOTEURS"); break;}
      case 16: {Serial.print("Mesure de CONTINUITE"); break;}
      case 17 : {Serial.print("CAPACIMETRE");} } }
      
void Affiche_le_mot_VOLTMETRE() {
  Serial.print("Voltm"); Serial.print(char(232)); Serial.print("tre");}
  
void Mesure_les_TENSIONS() {
  N_Tensions_cumulees = 0;
  for (int I = 0; I < (N) ; I++) 
      {Tester_le_BP_A0(); // Pour détection dans la boucle de mesure.
       Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;}     

void Affiche_les_TENSIONS() {     
  if (Valeur_calculee != Ancienne_Valeur) {      
      Ancienne_Valeur = Valeur_calculee;
      switch(Option_du_MENU) {
          case 1 : {Serial.print("Tension lue sur E = ");
                    Serial.println(Valeur_calculee/1000); break;}
          case 2 : {Serial.print("Tension lue sur 50Vcc = ");
                    Serial.println(Valeur_calculee/100,1); break;}
          case 3 : {Serial.print("Alternatif (3 a 30V) = ");
                    Serial.println(Valeur_calculee/100,1); break;}
          case 4 : {Serial.print("Alternatif (30 a 60V) = ");
                    Serial.println(Valeur_calculee/100,1);} } } }
  
void Fonction_GENERATEUR_PWM() {
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  Front_Montant = true; // variation PROGRADE en mode manuel.
  Option_du_MENU = 1; // Mode de génération en cours.
  Serial.println(); Serial.println("Fonction GENERATEUR PWM."); Serial.println();
  Clic_sur_le_BP = false; Clic_long = false; Augmenter_le_choix = false;
  Ancienne_PWM = 2000; // Forcer le premier affichage. 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  N_Tensions_cumulees = 0;
  for (int I = 0; I < (N) ; I++) 
      {Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;
  //=========== Traiter L'OPTION manuelles =============
  if (Option_du_MENU == 1) { 
      Valeur_calculee = map(Tension_lue, 0, 1023, 1, 254);
      if(!Front_Montant) Valeur_calculee = 254 - Valeur_calculee;
      if(Valeur_calculee != Ancienne_Valeur) {Ancienne_Valeur = Valeur_calculee;
      analogWrite(Sortie_PWM,Valeur_calculee);  // Modification du rapport cyclique.
      //================= AFFICHER ==================
      PAQUET = (int)Tension_lue; 
      if (Front_Montant) Serial.print("(Pro");
      else {Serial.print("(R"); Serial.print(char(233)); Serial.print("tro");}   
      Serial.print("rade ) Commande = ");
      if (PAQUET < 10)  Serial.print(" ");
      if (PAQUET < 100)  Serial.print(" ");
      if (PAQUET < 1000)  Serial.print(" ");
      Serial.print(PAQUET);
      Serial.print("  Valeur PWM = ");
      if (Valeur_calculee < 10)  Serial.print(" ");
      if (Valeur_calculee < 100)  Serial.print(" ");  
      Serial.print(Valeur_calculee,0);
      Serial.print("  Rapport cyclique = ");
      Serial.println(Valeur_calculee / 254,2); } }
      
  //=============== Traiter l'ESCALIER =================
  if (Option_du_MENU == 2) {Valeur_PWM = 128;
     analogWrite(Sortie_PWM,Valeur_PWM); // Impulsions PL+ moyenne.
     Serial.println(); Serial.println(" Mode ESCALIER en cours.");
     Afficher_PWM();
     Clic_sur_le_BP = false; Augmenter_le_choix = false; Clic_long = false;
     //--------- Tant que NON passage à RAMPE ----------
     while (!(Augmenter_le_choix)) {
       Tester_le_BP_A0();
       if ((Clic_sur_le_BP) && (!Augmenter_le_choix))  
         // Monter d'une marche. (11 marches comprises entre 3 et 253)
         {if (Valeur_PWM == 253) Valeur_PWM = 3; else Valeur_PWM = Valeur_PWM + 25; 
          analogWrite(Sortie_PWM,Valeur_PWM); Clic_sur_le_BP = false;
          Afficher_PWM(); }
     //------------- Passage au mode RAMPE -------------
       if (Augmenter_le_choix)
          {Clic_sur_le_BP = false; Option_du_MENU = 3;} } }
        
  //================ Traiter la RAMPE ================== 
  if ((Option_du_MENU == 3) && (!Clic_long)) {
     Afficher_mode_RAMPE();
     Clic_sur_le_BP = false; Augmenter_le_choix = false; Clic_long = false;
     //-------- Tant que NON retour à MANUEL ----------
     while (!(Augmenter_le_choix)) {
       //-------------- générer la RAMPE --------------
       if (Rapide) delay(10); else delay(30);
       Valeur_PWM++; if (Valeur_PWM == 255) Valeur_PWM = 1;
       analogWrite(Sortie_PWM,Valeur_PWM);
       //-------------- Traiter les B.P. --------------
       Tester_le_BP_A0();
       if ((Clic_sur_le_BP) && (!Augmenter_le_choix))  
         // Inverser le boolée Rapide.)
         {Rapide = !Rapide; Clic_sur_le_BP = false; Afficher_mode_RAMPE(); } 
     //------------- Retour au mode MANUEL -------------
     if (Augmenter_le_choix)
        {Clic_sur_le_BP = false; Option_du_MENU = 1;
        Serial.println(" Retour au mode manuel."); Serial.println();
     Ancienne_PWM = 2000;} } } // Forcer le premier affichage. 
  //====================================================    
  // traiter les deux B.P. dans la Fonction_GENERATEUR_PWM :
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     // B.P. augmenter le choix.
       if ((Augmenter_le_choix) && (!Clic_long))
          {Clic_sur_le_BP = false; Option_du_MENU = 2;}
     // B.P. diminuer le choix.
     if (!Augmenter_le_choix)
        {Front_Montant = !Front_Montant; Clic_sur_le_BP = false; } } }
  //=============================================================================
  analogWrite(Sortie_PWM, 128); // Génération PWM rétablie.
  Option_du_MENU = 1; }
  
void Afficher_PWM() {Serial.print(" Echelon PWM = "); Serial.println(Valeur_PWM);}

void Afficher_mode_RAMPE() {
  Serial.println(); Serial.print(" Mode RAMPE ");
  if (Rapide) Serial.print("RAPIDE"); else Serial.print("LENTE");
  Serial.println(" en cours."); }
      
void Fonction_PERIODEMETRE() {
  Serial.println(); Serial.println("Fonction PERIODEMETRE."); Serial.println();
  Clic_sur_le_BP = false;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  Valeur_cumulee = 0;
  TCCR1B = 0; // Bloquer VALEUR.
  //==================== MESURER ======================
  for (byte I=0; I<10; I++) { // Dix mesures pour lisser par calcul de moyenne.
    VALEUR = 0; Nb_IRQ=0; TCNT1 = 0; // Forcer le compteur rapide à zéro.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 0; // Bloquer VALEUR.
    // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
    VALEUR = TCNT1;
    // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
    VALEUR = TCNT1 + (65536 * Nb_IRQ);
    Valeur_cumulee = Valeur_cumulee + (VALEUR / 16);} 
    VALEUR = Valeur_cumulee / 10; // Calculer la moyenne des dix mesures. 
  //==================== AFFICHER =====================
  if (VALEUR != Ancienne_valeur) {Ancienne_valeur = VALEUR; Afficher_la_Periode();}    
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {if (!Augmenter_le_choix) PAUSE(); } }
  //=============================================================================
  Option_du_MENU = 1;
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir. 
  
void  Afficher_la_Periode() {  Serial.print(" P"); Serial.print(char (233));
  Serial.print("riode = "); Affiche_VALEUR_avec_formatage(); Serial.print(" ");
  Serial.print(char(181)); Serial.print("S soit une Fr"); Serial.print(char (233));
  Serial.print("quence = ");
//  if (VALEUR < 10000) Serial.print((float)(1000000 / VALEUR));
if (VALEUR <= 2000) Serial.print(1000000 / VALEUR); // QRG >= à 500Hz.
  else {VALEUR = 1000000000 / VALEUR; Affiche_VALEUR_avec_formatage();}
  Serial.println(" Hz.");
  if (VALEUR < 1000) {Serial.print("Pr"); Serial.print(char(233));
      Serial.print("f"); Serial.print(char(233));
      Serial.println("rer le FREQUENCEMETTRE.");} }
      
void Fonction_pilotage_des_SERVOMOTEURS() {
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  Pulse_plus = 1500; // Neutre. 
  Serial.println(); Serial.println("Pilotage de SERVOMOTEURS."); Serial.println();
  Clic_sur_le_BP = false; Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
   Piloter();
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     if ((Augmenter_le_choix) && (!Clic_long))
        { Clic_sur_le_BP = false;
          if (Pulse_plus < 3000) Pulse_plus = Pulse_plus + 100;}
     if (!Augmenter_le_choix)
        { Clic_sur_le_BP = false;
          if (Clic_long) Pulse_plus = 1500;
          else {if (Pulse_plus > 200) Pulse_plus = Pulse_plus - 100; } } } }
  //=============================================================================
  analogWrite(Sortie_PWM, 128); // Génération PWM rétablie.
  Option_du_MENU = 1; }
  
void Piloter() {
  //==================== PILOTER ======================
  digitalWrite(Sortie_SIGNAL, HIGH);
  delayMicroseconds(Pulse_plus);
  digitalWrite(Sortie_SIGNAL, LOW);
  delay(15); // Délai constant pour commencer le 20mS.
  delayMicroseconds(5000 - Pulse_plus); // Ajustement pour une fréquence constante.
  //==================== AFFICHER =====================
  if (Pulse_plus != Ancienne_valeur) {
      Ancienne_valeur = Pulse_plus;
      Serial.println();
      Serial.print(" Travail = "); Serial.print(Pulse_plus);
      Afficher_microS();
      Serial.print(" Repos = "); Serial.print(20000 - Pulse_plus);
      Afficher_microS(); Serial.println();} }
        
void Fonction_CAPACIMETRE() {
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  Serial.println(); Serial.println("Fonction CAPACIMETRE."); Serial.println();
  Clic_sur_le_BP = false; Ancienne_Valeur = 12345; // Forcer l'affichage.
  TCCR1A = 0;   // Le Timer 1 est configuré en simple VALEUR.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  //==================== MESURER ======================
  TCCR1B = 0; // Bloquer VALEUR.
  RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant.
  Valeur_calculee = 0; Pulse_plus = 0; // Pulse_plus mesure A1.
  digitalWrite(Sortie_SIGNAL,HIGH); // D12 passe à +5vcc.
  TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
  while (Pulse_plus < 644) {Pulse_plus = analogRead(1);}
  TCCR1B = 0; // Bloquer VALEUR.
  digitalWrite(Sortie_SIGNAL,LOW); // D12 repasse à zéro pour décharger le condensateur.
  VALEUR = TCNT1;
  // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
  VALEUR = TCNT1 + (65536 * Nb_IRQ);
  Valeur_calculee = VALEUR / 4450;
  //==================== AFFICHER =====================
  if (Pulse_plus > 1000) Valeur_calculee = 0; // Afficher zéro si pas de condensateur.
  if ((Valeur_calculee) != Ancienne_Valeur)
     {Ancienne_Valeur = Valeur_calculee; Afficher_condensateur();}
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     if (!Augmenter_le_choix) {Clic_sur_le_BP = false; PAUSE();
         Ancienne_Valeur = 12345; } } // Forcer l'affichage en sortie de PAUSE.
  // Ci-dessous assurer la décharge complète du condensateur.
  while (Pulse_plus > 10) {Pulse_plus = analogRead(1);} }
  //=============================================================================
  Option_du_MENU = 1; analogWrite(Sortie_PWM, 128); // Génération PWM rétablie.
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir.
  
void Afficher_condensateur() {
  Serial.print("Capacit"); Serial.print(char(233));
  Serial.print(" = "); Serial.print(Valeur_calculee,0);
  Afficher_micro(); Serial.println("F."); }
      
void Fonction_IMPULSIOMETRE() {
  Serial.println(); Serial.println("Fonction IMPULSIOMETRE."); Serial.println();
  Clic_sur_le_BP = false;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple VALEUR.
  // ============== Autoriser les interruptions sur débordement =================
  TIMSK1 |= (1<<TOIE1);
  Clic_long = false; Augmenter_le_choix = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //============================ Tant que non SORTIE ============================
  //==================== MESURER ======================
  Valeur_calculee = 0;
  for (byte I=0; I<10; I++) {
    TCCR1B = 0; // Bloquer VALEUR.
    RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Attendre un état "1".
    while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
    TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
    // Attendre un état "0".
    while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
    // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
    TCCR1B = 0; // Bloquer VALEUR.
    VALEUR = TCNT1;
    // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
    VALEUR = TCNT1 + (65536 * Nb_IRQ);
    Valeur_calculee = Valeur_calculee + VALEUR / 16; }
    Pulse_plus = VALEUR / 16;
    Valeur_calculee = 0;
    for (byte I=0; I<10; I++) {
      TCCR1B = 0; // Bloquer VALEUR.
      RAZ_VALEUR(); // Initialise provisoirement pour source sur D5 et front montant. 
      // Attendre un état "1".
      while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
      // Attendre un état "0".
      while (digitalRead(Entree_SIGNAL) == 1) {}; // Tant que "1" ne rien faire.
      TCCR1B = 1; // Horloge XTAL comptée sans prédiviseur.
      // Attendre un état "1".
      while (digitalRead(Entree_SIGNAL) == 0) {}; // Tant que "0" ne rien faire.
      // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
      TCCR1B = 0; // Bloquer VALEUR.
      VALEUR = TCNT1;
      // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
      VALEUR = TCNT1 + (65536 * Nb_IRQ);
      Valeur_calculee = Valeur_calculee + VALEUR / 16; }
    Pulse_moins = VALEUR / 16;
  //==================== AFFICHER =====================
  if ((Pulse_moins + Pulse_plus) != Ancienne_valeur)
     {Ancienne_valeur = Pulse_moins + Pulse_plus; Afficher_Impulsion();}
  //================ Traiter les B.P. =================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
      if (!Augmenter_le_choix) PAUSE(); } }
  //=============================================================================
  Option_du_MENU = 1; TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir. 
  
void RAZ_VALEUR() {
  VALEUR = 0; Ancienne_valeur = 1; // Pour forcer l'affichage; 
  // Replacer le VALEUR à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0; 
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);}
  
  void Afficher_Impulsion() {
  Serial.print(" PL+ = "); Serial.print(Pulse_plus); Serial.print(char(181));
  Serial.print("S PL- = "); Serial.print(Pulse_moins); Serial.print(char(181));
  Serial.print("S T = "); Serial.print(Pulse_moins + Pulse_plus);
  Serial.print(char(181)); Serial.print("S QRG = ");
  Serial.print(1000000 / (Pulse_moins + Pulse_plus));
  Rapport_cyclique = ((float)Pulse_plus /(Pulse_moins + Pulse_plus));
  Serial.print("Hz PL+/T = "); Serial.println(Rapport_cyclique,2); }
      
void Fonction_TESTEUR_de_CONTINUITE() {
  // L'instruction tone() utilise le TIMER2 donc pas de génération sur PWM11.
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  N = 100; // Diminuer le nombre de mesure pour augmenter la rapidité.
  boolean Debit_50mA = false; // Test à "forte intensité". (50mA sur CC.)
  //-------------- Initialiser pour afficher A2 utilisée --------------
  Clic_long = false; Augmenter_le_choix = false; Clic_sur_le_BP = true;
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE ==========================
  Tester_le_BP_A0();
  //-------------- Traiter les deux B.P. --------------
  if (Clic_sur_le_BP) {
     Clic_sur_le_BP = false; Serial.println(); Serial.print(" Mesure sur ");
     if (Augmenter_le_choix) {Debit_50mA = true; Serial.print("A3 (50mA)");}
        else {Debit_50mA = false; Serial.print("A2 (5mA)");} }
  //---------------------------------------------------
  // Mesurer la continuité sur A2 ou A3.
  if (Debit_50mA) N_mesures_sur_Ai(3); // Débit Max 50mA.
     else N_mesures_sur_Ai(2); // Débit Max 5mA.
  // Générer la tonalité.
  // 454 : résistance > 100 ohm. (Silence)
  // 327 à 454 : résistance entre 70 et 100 ohm. (150 Hz)
  // 192 à 327 : résistance entre 40 et 70 ohm. (300 Hz)
  // 98 à 192 : résistance entre 20 et 40 ohm. (800 Hz)
  // 75 à 98 : résistance entre 10 et 20 ohm. (300 Hz)
  // <75 : résistance < 10 ohm. (2000 Hz)
  if (Tension_lue > 454) noTone(Sortie_SIGNAL); // R > 100 ohm.
  if ((Tension_lue < 455) && (Tension_lue > 327)) tone(Sortie_SIGNAL,50);
  if ((Tension_lue < 328) && (Tension_lue > 192)) tone(Sortie_SIGNAL,75);
  if ((Tension_lue < 193) && (Tension_lue > 98)) tone(Sortie_SIGNAL,300);
  if ((Tension_lue < 99) && (Tension_lue > 75)) tone(Sortie_SIGNAL,800);
  if (Tension_lue < 76) tone(Sortie_SIGNAL,2000); }
  //========================================================================
  noTone(Sortie_SIGNAL); analogWrite(Sortie_PWM, 128); // Génération PWM rétablie.
  N = 400;} // Restituer la valeur de départ.
      
void Fonction_GENERATEUR_BF() {
  // L'instruction tone() utilise le TIMER2 donc pas de génération sur PWM11.
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  N = 1000; // Augmenter le nombre de mesures pour faire la moyenne.
  long QRG; // Fréquence générée sur D12.
  byte Plage_QRG = 2; // Plage de fréquences actuellement générées.
  Clic_long = false; Augmenter_le_choix = true; Clic_sur_le_BP = false;
  //========================== Tant que non SORTIE ========================== 
  while (!(Augmenter_le_choix && Clic_long)) {
    Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].  
      N_Tensions_cumulees = 0;
      for (int I = 0; I < (N) ; I++) 
          {Tension_lue = analogRead(1); // Mesure analogique [0 à 1023].
          N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
    Tension_lue = N_Tensions_cumulees / N;
  //============ Traiter les BP =============
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     if (!Clic_long) 
        { Clic_sur_le_BP = false;
          if (Augmenter_le_choix)
             {if (Plage_QRG < 4) Plage_QRG++; else Plage_QRG = 1;}
          else {if (Plage_QRG > 1) Plage_QRG--; else Plage_QRG = 4;} } }
  // =============== M.A.J. la plage de variation ===============
  //Calculer la valeur de la tension lue sur l'entrée du multimètre.
  switch (Plage_QRG) {
    case 1 : {QRG = map(Tension_lue, 15, 1000, 31, 200); break;} // Fréquence en Hz.
    case 2 : {QRG = map(Tension_lue, 11, 1005, 150, 2000); break;} // Fréquence en Hz.
    case 3 : {QRG = map(Tension_lue, 10, 1005, 1500, 10000); break;} // Fréquence en Hz.
    case 4 : {QRG = map(Tension_lue, 9, 1010, 9000, 50000); } } // Fréquence en Hz.
  if (QRG != Ancienne_Valeur) {      
      Ancienne_Valeur = QRG; tone(Sortie_SIGNAL,QRG);
      switch (Plage_QRG) {
        case 1 : {Serial.print("[31 a 200] "); break;}
        case 2 : {Serial.print("[150 a 2000] "); break;}
        case 3 : {Serial.print("[1500 a 10000] "); break;}
        case 4 : {Serial.print("[9000 a 50000] "); } }
      Serial.print("Fr"); Serial.print(char (233)); Serial.print("quence fournie = ");
      VALEUR = QRG; Affiche_VALEUR_avec_formatage(); Serial.println(); } }
   //========================================================================
  noTone(Sortie_SIGNAL); analogWrite(Sortie_PWM, 128); // Génération PWM rétablie.
  N = 400;} // Restituer la valeur de départ.}
      
void Fonction_ETALON_de_QRG() {
  // L'instruction tone() utilise le TIMER2 donc pas de génération sur PWM11.
  pinMode(Sortie_PWM, INPUT); // Stoppe la PWM.
  QRG_ETALON = 10; Afficher_QRG_produite();
  Clic_long = false; Augmenter_le_choix = true; Clic_sur_le_BP = false; 
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE ==========================
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {
     if (!Clic_long) 
        { Clic_sur_le_BP = false;
          if (Augmenter_le_choix)
             {if (QRG_ETALON < MAX_QRG_ETALON) QRG_ETALON++; else QRG_ETALON = 1;}
          else {if (QRG_ETALON > 1) QRG_ETALON--; else QRG_ETALON = MAX_QRG_ETALON;}
        Afficher_QRG_produite();} } }
  //========================================================================
  noTone(Sortie_SIGNAL); analogWrite(Sortie_PWM, 128); } // Génération PWM rétablie.
        
void Afficher_QRG_produite() {
  Serial.print("Fr"); Serial.print(char (233));Serial.print("quence fournie = ");
  switch (QRG_ETALON) {
    case 1 : {Serial.print("50"); tone(Sortie_SIGNAL,50); break;}
    case 2 : {Serial.print("75"); tone(Sortie_SIGNAL,75); break;}
    case 3 : {Serial.print("100"); tone(Sortie_SIGNAL,100); break;}
    case 4 : {Serial.print("150"); tone(Sortie_SIGNAL,150); break;}
    case 5 : {Serial.print("200"); tone(Sortie_SIGNAL,200); break;}
    case 6 : {Serial.print("250"); tone(Sortie_SIGNAL,250); break;}
    case 7 : {Serial.print("300"); tone(Sortie_SIGNAL,300); break;}
    case 8 : {Serial.print("400"); tone(Sortie_SIGNAL,400); break;}
    case 9 : {Serial.print("500"); tone(Sortie_SIGNAL,500); break;}
    case 10 : {Serial.print("1000"); tone(Sortie_SIGNAL,1000); break;}
    case 11 : {Serial.print("2000"); tone(Sortie_SIGNAL,2000); break;}
    case 12 : {Serial.print("4000"); tone(Sortie_SIGNAL,4000); break;}
    case 13 : {Serial.print("8000"); tone(Sortie_SIGNAL,8000); break;}
    case 14 : {Serial.print("10000"); tone(Sortie_SIGNAL,10000); break;}
    case 15 : {Serial.print("20000"); tone(Sortie_SIGNAL,20000); break;}
    case 16 : {Serial.print("40000"); tone(Sortie_SIGNAL,40000); break;}
    case 17 : {Serial.print("50000"); tone(Sortie_SIGNAL,50000); break;}
    case 18 : {Serial.print("LA (3) 440"); tone(Sortie_SIGNAL,440); } }
  Serial.println(" Hz");}
    
void Fonction_COMPTEUR() {
  Clic_sur_le_BP = false;
  Serial.println(); Serial.println("FONCTION COMPTEUR en cours.");
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);
  // Autorise les interruptions sur débordement.
  // (Active la procédure ISR(TIMER1_OVF_vect) sur débordement.)
  TIMSK1 |= (1<<TOIE1);
  Clic_sur_le_BP = false;
  RAZ_COMPTEUR(); Augmenter_le_choix = false;
  while (!(Augmenter_le_choix && Clic_long)) {
        Tester_le_BP_A0();
        if (Clic_sur_le_BP) {
           Clic_sur_le_BP = false;
           if (Augmenter_le_choix) 
               {if(!Clic_long) {
                 Front_Montant = !Front_Montant;
                  Ancienne_Valeur++; // Pour forcer l'affichage; 
                  if (Front_Montant) TCCR1B = TCCR1B | (1<<CS10);
                  else  TCCR1B = 6; } }
           else RAZ_COMPTEUR(); }
  //========================= Traiter le VALEUR ===============================
  // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
  VALEUR = TCNT1;
  // Lecture des 16 bits dans la variable VALEUR et ajoute le DEBORDEMENT.
  VALEUR = TCNT1 + (65536 * Nb_IRQ);
  if (VALEUR > 9999999)
     {VALEUR = 9999999; TCCR1B = 0;} // Bloquer le comteur.
  // ==========================================================================
  if(VALEUR != Ancienne_Valeur) { 
  Affiche_VALEUR_avec_formatage(); Serial.print(" Fronts "); 
  if (Front_Montant) Serial.println("+"); else Serial.println("-");
  Ancienne_Valeur = VALEUR;} }
  // ===================== Sortie de la fonction COMPTEUR =====================
  Option_du_MENU = 1;}

void RAZ_COMPTEUR() {
  VALEUR = 0; Ancienne_Valeur = 1; // Pour forcer l'affichage;
  TCCR1B = 7; // Autorise les comptages sur fronts montants.
  Front_Montant = true;  
  TCNT1 = 0; Nb_IRQ = 0;} // Replacer le compteur à zéro.
    
void Fonction_FREQUENCEMETRE() {
  Clic_long = false; Rapide = true; Clic_sur_le_BP = false;
  Ancienne_valeur = 0;
  TCCR1A = 0;   // Le Timer 1 est configuré en simple compteur.
  // 111 pour Source externe sur D5 et comptage sur le front montant :
  TCCR1B = (1<<CS12) | (1<<CS11) | (1<<CS10);	
  // Autorise les interruptions sur débordement.
  // (Active la procédure ISR(TIMER1_OVF_vect) sur débordement définie ci-avant.)
  TIMSK1 = (1<<TOIE1); 
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE ========================= 
  Tester_le_BP_A0();
  //-------------- Traiter les deux B.P. --------------
  // Le B.P. augmenter le choix n'est utilisé que pour sortir car en
  // mesures à 1Hz il n'est pas pris en compte 9 fois sur 10.
  if ((Clic_sur_le_BP) && (!Augmenter_le_choix))
     {Clic_sur_le_BP = false;
      if ((Rapide) && (Clic_long))
          {PAUSE(); Ancienne_valeur = 0;}
      else Rapide = !Rapide;}
  //---------------------------------------------------    
  Mesurer_la_frequence();
  if (VALEUR != Ancienne_valeur) { Ancienne_valeur = VALEUR; Afficher_la_frequence();} }
  //========================================================================
  TCCR1B = 0; } // Bloquer les IRQ du TIMER 1 avant de sortir.

void  Mesurer_la_frequence() { 
  digitalWrite(LED_Arduino, !digitalRead(LED_Arduino)); // Inverser la LED 13.
  if (Rapide) {Temps_de_mesure = 99;  Affiner_T = 986;}
     else {Temps_de_mesure = 999; Affiner_T = 978;}
  // Replacer le compteur à zéro pour une nouvelle mesure.
  TCNT1 = 0x0000; // Remettre le TIMER1 à zéro, autre écriture possible : TCNT1 = 0;
  Nb_IRQ=0;
  // Par définition, la fréquence correspond aux nombres d'impulsions par seconde.
  // Pendant ce délai il y a comptage sur Timer 1 et éventuellement plusieurs débordements.
  //============= Calibrer la durée de mesures ===============
  delay(Temps_de_mesure); 
  delayMicroseconds(Affiner_T);
  // =========================================================  
  // Ci-dessous une première lecture fait passer TCNT1H dans le TAMPON.
  VALEUR = TCNT1;
  // Lecture des 16 bits dans la variable COMPTEUR et ajoute le DEBORDEMENT.
  VALEUR = TCNT1 + (65536 * Nb_IRQ);
  // =========================================================
  if(Rapide) VALEUR = VALEUR * 10;}
  
void  Afficher_la_frequence() {
  Afficher_L_ou_R();  
  Serial.print(" Fr"); Serial.print(char (233)); Serial.print("quence = ");
  Affiche_VALEUR_avec_formatage(); Serial.print(" Hz soit une periode de : ");
  if (VALEUR > 0) VALEUR = 1000000 / VALEUR; else VALEUR = 0; // Attention à la division par zéro !
  Serial.print(VALEUR); Afficher_microS(); Serial.println(); }
  
void Afficher_microS() {Afficher_micro(); Serial.print("S"); }
  
void Afficher_micro() {Serial.print(" "); Serial.print(char(181));}

void Afficher_L_ou_R() {if (Rapide) Serial.print("(R) "); else Serial.print("(L) ");}
  
void COMPLETER() {if (ESPACE) {Serial.print(" ");} else {Serial.print("0");} } 
  
void Afficher_un_PAQUET() {// Valeur comprise entre 0 et 999.
  if (PAQUET < 100) {COMPLETER();} // Afficher un espace ou un zéro.
  if (PAQUET < 10) {COMPLETER();}  // Afficher un autre espace ou un autre zéro.
  if (PAQUET == 0) {COMPLETER();} else {Serial.print(PAQUET);} }

void Affiche_VALEUR_avec_formatage() {
  // Formate l'affichage pour placer les points et les espaces en tête pour
  // 9 caractères. Ne modifie pas la variable VALEUR.
  unsigned long SAV_VALEUR = VALEUR; boolean Mega_non_nul;
  ESPACE = true; // Commencer par des espaces si valeur insuffisante.
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les MEGA <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000000; // Isoler les Méga dans PAQUET et VALEUR inchangée.
  // Si > 0 afficher les Méga suivi d'un point, si non afficher deux espaces.
  if (PAQUET == 0) {Serial.print("  "); Mega_non_nul = false;}
      else {Serial.print(PAQUET); Serial.print("."); ESPACE = false; Mega_non_nul = true;
            // Enlever les MEGA déjà affichés dans VALEUR :
            VALEUR = VALEUR - (PAQUET * 1000000); }
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les KILO <<<<<<<<<<<<<<<<<<<<<<<<<<<*/
  PAQUET = VALEUR / 1000; // Isoler les KILO dans PAQUET et VALEUR inchangée.
  Afficher_un_PAQUET();
  if ((PAQUET > 0) | (Mega_non_nul))
     {Serial.print("."); ESPACE = false;} else {Serial.print(" "); } 
  /*>>>>>>>>>>>>>>>>>>>>>>>>>>> Afficher les UNITÉS <<<<<<<<<<<<<<<<<<<<<<<<<<*/
  // Enlever les KILO déjà affichés à VALEUR.
  if (PAQUET > 0) {PAQUET = VALEUR - (PAQUET * 1000);} else {PAQUET = VALEUR;}
  if ((PAQUET == 0) && (ESPACE)) {Serial.print("  0");} else {Afficher_un_PAQUET();}
  VALEUR = SAV_VALEUR; }
                               
void Fonction_CHRONOMETRE() {
  Clic_long = false; Chrono_ON = false; Bloquer_a_24H = false;
  Cadence_de_comptage = 1000; RAZ_CHRONOMETRE();
  while (!(Augmenter_le_choix && Clic_long)) {
  //========================== Tant que non SORTIE =========================
  if ((!Bloquer_a_24H) && (Chrono_ON)) Incrementer_le_CHRONOMETRE();
  Tester_le_BP_A0();
  //-------------- Traiter les deux B.P. --------------
  if (Clic_sur_le_BP) {
     Clic_sur_le_BP = false;
     // B.P. augmenter le choix.
       if (Augmenter_le_choix)
          {Chrono_ON = !Chrono_ON; if (Chrono_ON)
               Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis()+ Cadence_de_comptage; }
     // B.P. diminuer le choix.
       else {if (Clic_long)
                    if (Cadence_de_comptage == 1000) Cadence_de_comptage = 100;
                    else Cadence_de_comptage = 1000;
             else RAZ_CHRONOMETRE(); } } }
  //========================================================================
  Option_du_MENU = 1; }
 
void Traite_Clic_LONG() {
  Serial.print("Clic LONG et cadence = "); Serial.println(Cadence_de_comptage);
  if (Cadence_de_comptage == 1000) Cadence_de_comptage = 10;
  else Cadence_de_comptage = 100;}
 
void Incrementer_le_CHRONOMETRE() {
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = 
      Ancienne_valeur_Temps_Ecoule_depuis_RESET + Cadence_de_comptage;
      if (Cadence_de_comptage == 100) DIXIEMES++; else SECONDES++;
      digitalWrite(LED_Arduino,!digitalRead(LED_Arduino)); // Inverser LED 13.
      if (DIXIEMES == 10) {SECONDES++; DIXIEMES = 0;}
      if (SECONDES == 60) {MINUTES++; SECONDES = 0;}
      if (MINUTES == 60) {HEURES++; MINUTES = 0;}
      Bloquer_a_24H = (HEURES == 24); Afficher_le_CHRONOMETRE(); } }
 
void RAZ_CHRONOMETRE() {
  HEURES = 0; MINUTES = 0; SECONDES = 0; DIXIEMES = 0; 
  Ancienne_valeur_Temps_Ecoule_depuis_RESET = 
    Ancienne_valeur_Temps_Ecoule_depuis_RESET + Cadence_de_comptage;
  Afficher_le_CHRONOMETRE();}
        
void  Afficher_le_CHRONOMETRE() {
  Impose_deux_caracteres(HEURES); Serial.print(HEURES); Serial.print(" H ");
  Impose_deux_caracteres(MINUTES); Serial.print(MINUTES); Serial.print(" Min ");
  Impose_deux_caracteres(SECONDES); Serial.print(SECONDES); Serial.print(" S ");
  if (Cadence_de_comptage == 1000) Serial.println(); else Serial.println(DIXIEMES); }
  
void Impose_deux_caracteres(byte VALEUR) {if (VALEUR < 10) {Serial.print(" ");} }

void Mesure_et_affiche_les_INTENSITES() {
  // Mesures sur A4 avec résistance shunt de 10 ohms.
  N_mesures_sur_Ai(4); Entree_A4 = true;
  Valeur_calculee = Tension_lue * K10 / 10;
  if ((Valeur_calculee < 50) && (Valeur_calculee > 0)) {Affiche_Changer_de_calibre();}
  // Si zéro sur A4 mesurer sur A5 avec résistance shunt de 100 ohms.
  if (Valeur_calculee == 0) {
      N_mesures_sur_Ai(5); Entree_A4 = false;
      Valeur_calculee = Tension_lue * K100 / 100;}
  Affiche_Intensite(); }

void Mesure_et_affiche_les_RESISTANCES() {
  // Mesures sur A2 avec résistance shunt de 1000 ohms.
  N_mesures_sur_Ai(2);
  Valeur_calculee = Tension_lue * 1000 / (5000 - Tension_lue); 
  if (Valeur_calculee < 200) {Affiche_Changer_de_calibre();}
  if (Tension_lue < 4960)
     {Entree_A2 = true; Affiche_la_resistance();} // N'affiche que si changée.
  // Mesures sur A3 avec résistance de limitation de 100 ohms.
  N_mesures_sur_Ai(3);
  Valeur_calculee = Tension_lue * 100 / (5000 - Tension_lue);
  if (Tension_lue < 4960)
     { Entree_A2 = false; Affiche_la_resistance();} } // N'affiche que si changée.

void Affiche_la_resistance() {  
  if (Valeur_calculee != Ancienne_Valeur) {
  Ancienne_Valeur = Valeur_calculee;
  Serial.print("  Tension lue = ");
  Serial.print(Tension_lue/1000,2);
  Serial.print("   R"); Serial.print(char (233)); Serial.print("sistance sur ");
  if (Entree_A2) {Serial.print("A2");} else {Serial.print("A3");}
  Serial.print(" = ");   Serial.println(Valeur_calculee,1); } }

void N_mesures_sur_Ai(byte ENTREE) { 
  N_Tensions_cumulees = 0;
  for (int I = 0; I < N ; I++) 
      {Tester_le_BP_A0(); // Pour détection dans la boucle de mesure.
       Tension_lue = analogRead(ENTREE); // Mesure analogique [0 à 1023].
       N_Tensions_cumulees = N_Tensions_cumulees + Tension_lue; }
  Tension_lue = N_Tensions_cumulees / N;
  Tension_lue = map(Tension_lue, 0, 1023, 0, 5000); // Tension en volts * 1000.
  if (Tension_lue == 5000) {Tension_lue = 4999.99;} }
  // 4999.99 au lieu de 5000 pour viter une division par zéro dans le calcul de R.
 
 void Affiche_le_mot_Intensite() {Serial.print("Intensit"); Serial.print(char (233));}
 
 void Affiche_Intensite() {  
  if (Valeur_calculee != Ancienne_Valeur) {
  Ancienne_Valeur = Valeur_calculee;
  Serial.print("Tension lue sur ");
  if (Entree_A4) {Serial.print("A4");} else {Serial.print("A5");}
  Serial.print(" = "); Serial.print(Tension_lue/1000);
  Serial.print("  >>>  "); Affiche_le_mot_Intensite(); Serial.print(" = ");
  if (Entree_A4) {Serial.print(Valeur_calculee,0);} else {Serial.print(Valeur_calculee,1);}
  Serial.println(" mA"); } }

void Affiche_Changer_de_calibre() { Serial.println(); Serial.println();
    Serial.println("  Changer de calibre."); Serial.println(); delay(250); }  
    
void CONFIRMER() { // Si non confirmé fait croire à une demande d'option voisine.
  boolean Mode_actuel = Augmenter_le_choix; // Mémorise le sens actuel de cheminement.
  Serial.print(" ? --> Confirmez : ");
  while (!Clic_sur_le_BP) Tester_le_BP_A0(); // Attente d'une action sur l'un des deux B.P.
  Confirmation = Augmenter_le_choix; // Confirmé si B.P. d'option prograde activé.
  Augmenter_le_choix = Mode_actuel;
  if (Confirmation) Serial.println("OUI"); else Serial.println("NON"); }
  
void PAUSE() {
  Clic_sur_le_BP = false; Serial.println();
  Serial.println("PAUSE en cours.");
  Serial.print("B.P. [Fonction moins] pour reprendre : ");
  while ((!Clic_sur_le_BP) || ( Augmenter_le_choix)) Tester_le_BP_A0();
  Clic_sur_le_BP = false; Serial.println("Sortie de la PAUSE.");}
  
 // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Arduino, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    delay(100); // Attendre_la_stabilisation du travail.
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) { COMPTEUR = 0; } }
    digitalWrite(LED_Arduino, LOW); // Éteindre la LED verte.
    if ((millis() - Horloge_actuelle) > 500) Clic_long = true; } }
  
